==> racer-0.0.4.tgz: package/lib/PubSub.js <==
var PubSub, RedisAdapter, handlePaths, hasKeys, pathParser, redis, transaction;
redis = require('redis');
pathParser = require('./pathParser.server');
transaction = require('./transaction.server');
hasKeys = require('./util').hasKeys;
PubSub = module.exports = function(options) {
  var adapterName, onMessage;
  if (options == null) {
    options = {};
  }
  adapterName = options.adapter || 'Redis';
  onMessage = options.onMessage || function() {};
  this._adapter = new PubSub._adapters[adapterName](onMessage, options);
};
PubSub.prototype = {
  subscribe: function(subscriberId, paths, callback) {
    return this._adapter.subscribe(subscriberId, paths, callback);
  },
  publish: function(path, message) {
    return this._adapter.publish(path, message);
  },
  unsubscribe: function(subscriberId, paths, callback) {
    return this._adapter.unsubscribe(subscriberId, paths, callback);
  },
  hasSubscriptions: function(subscriberId) {
    return this._adapter.hasSubscriptions(subscriberId);
  },
  subscribedToTxn: function(subscriberId, txn) {
    return this._adapter.subscribedToTxn(subscriberId, txn);
  }
};
PubSub._adapters = {};
PubSub._adapters.Redis = RedisAdapter = function(onMessage, options) {
  var db, event, host, makeCallback, port, ropts, subClient, subs, _fn, _i, _len, _onMessage, _ref, _ref2, _ref3;
  if (!(this._publishClient = options.pubClient)) {
    ropts = (_ref = options.redis || {}, port = _ref.port, host = _ref.host, db = _ref.db, _ref);
    this._publishClient = redis.createClient(port, host, ropts);
    if (db) {
      this._publishClient.select(db);
    }
  }
  if (!(this._subscribeClient = subClient = options.subClient)) {
    ropts = (_ref2 = options.redis || {}, port = _ref2.port, host = _ref2.host, db = _ref2.db, _ref2);
    this._subscribeClient = redis.createClient(port, host, ropts);
    if (db) {
      this._subscribeClient.select(db);
    }
  }
  this._subs = subs = {};
  this._subscriberSubs = {};
  if (options.debug) {
    _ref3 = ['subscribe', 'unsubscribe', 'psubscribe', 'punsubscribe'];
    _fn = function(event) {
      return subClient.on(event, function(path, count) {});
    };
    for (_i = 0, _len = _ref3.length; _i < _len; _i++) {
      event = _ref3[_i];
      _fn(event);
    }
    subClient.on('message', function(channel, message) {
      return console.log("MESSAGE " + channel + " " + message);
    });
    subClient.on('pmessage', function(pattern, channel, message) {
      return console.log("PMESSAGE " + pattern + " " + channel + " " + message);
    });
    this.__publish = RedisAdapter.prototype.publish;
    this.publish = function(path, message) {
      console.log("PUBLISH " + path + " " + (JSON.stringify(message)));
      return this.__publish(path, message);
    };
  }
  _onMessage = function(glob, path, message) {
    var key, re, subscriberId, subscribers, _ref4, _results;
    message = JSON.parse(message);
    if (subscribers = subs[glob]) {
      _results = [];
      for (key in subscribers) {
        _ref4 = subscribers[key], subscriberId = _ref4[0], re = _ref4[1];
        _results.push(re.test(path) ? onMessage(subscriberId, message) : void 0);
      }
      return _results;
    }
  };
  subClient.on('message', function(path, message) {
    return _onMessage(path, path, message);
  });
  subClient.on('pmessage', _onMessage);
  makeCallback = function(queue, events) {
    var event, fn, _j, _len2, _results;
    fn = function(path) {
      var callback, pending;
      if (pending = queue[path]) {
        if (callback = pending.shift()) {
          if (!--callback.__count) {
            return callback();
          }
        }
      }
    };
    _results = [];
    for (_j = 0, _len2 = events.length; _j < _len2; _j++) {
      event = events[_j];
      _results.push(subClient.on(event, fn));
    }
    return _results;
  };
  makeCallback(this._pendingSubscribe = {}, ['subscribe', 'psubscribe']);
  makeCallback(this._pendingUnsubscribe = {}, ['unsubscribe', 'punsubscribe']);
};
RedisAdapter.prototype = {
  subscribe: function(subscriberId, paths, callback) {
    var glob, globSubs, path, re, ss, subs, subscriberSubs, toAdd, _i, _len;
    if (subscriberId === void 0) {
      return;
    }
    subs = this._subs;
    subscriberSubs = this._subscriberSubs;
    toAdd = [];
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      path = paths[_i];
      glob = pathParser.glob(path);
      if (!(globSubs = subs[glob])) {
        subs[glob] = globSubs = {};
        toAdd.push(glob);
      }
      re = pathParser.regExp(path);
      globSubs["" + subscriberId + "$" + path] = [subscriberId, re];
      subscriberSubs[subscriberId] = ss = subscriberSubs[subscriberId] || {};
      ss[path] = re;
    }
    return handlePaths(toAdd, this._pendingSubscribe, this._subscribeClient, 'subscribe', 'psubscribe', callback);
  },
  publish: function(path, message) {
    return this._publishClient.publish(path, JSON.stringify(message));
  },
  unsubscribe: function(subscriberId, paths, callback) {
    var glob, globSubs, path, ss, subs, subscriberSubs, toRemove, _i, _len;
    if (subscriberId === void 0) {
      return;
    }
    if (typeof paths === 'function') {
      callback = paths;
      paths = null;
    }
    subscriberSubs = this._subscriberSubs;
    paths || (paths = subscriberSubs[subscriberId] || []);
    subs = this._subs;
    toRemove = [];
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      path = paths[_i];
      glob = pathParser.glob(path);
      if (globSubs = subs[glob]) {
        delete globSubs["" + subscriberId + "$" + path];
        if (!hasKeys(globSubs)) {
          toRemove.push(glob);
        }
      }
      ss = subscriberSubs[subscriberId];
      if (ss) {
        delete ss[path];
      }
    }
    return handlePaths(toRemove, this._pendingUnsubscribe, this._subscribeClient, 'unsubscribe', 'punsubscribe', callback);
  },
  hasSubscriptions: function(subscriberId) {
    return subscriberId in this._subscriberSubs;
  },
  subscribedToTxn: function(subscriberId, txn) {
    var p, path, re, _ref;
    path = transaction.path(txn);
    _ref = this._subscriberSubs[subscriberId];
    for (p in _ref) {
      re = _ref[p];
      if (p === path || re.test(path)) {
        return true;
      }
    }
    return false;
  }
};
handlePaths = function(paths, queue, client, pathFn, patternFn, callback) {
  var i, path, pending, _results;
  if (i = paths.length) {
    if (callback) {
      callback.__count = i;
    }
  } else {
    if (callback) {
      callback();
    }
  }
  _results = [];
  while (i--) {
    path = paths[i];
    if (pathParser.isGlob(path)) {
      client[patternFn](path);
    } else {
      client[pathFn](path);
    }
    _results.push(callback ? (queue[path] = pending = queue[path] || [], pending.push(callback)) : void 0);
  }
  return _results;
};

==> racer-0.0.8.tgz: package/lib/PubSub.js <==
var PubSub, RedisAdapter, handlePaths, hasKeys, pathParser, redis, transaction;
redis = require('redis');
pathParser = require('./pathParser.server');
transaction = require('./transaction.server');
hasKeys = require('./util').hasKeys;
PubSub = module.exports = function(options) {
  var adapterName, onMessage;
  if (options == null) {
    options = {};
  }
  adapterName = options.adapter || 'Redis';
  onMessage = options.onMessage || function() {};
  this._adapter = new PubSub._adapters[adapterName](onMessage, options);
};
PubSub.prototype = {
  subscribe: function(subscriberId, paths, callback) {
    return this._adapter.subscribe(subscriberId, paths, callback);
  },
  publish: function(path, message) {
    return this._adapter.publish(path, message);
  },
  unsubscribe: function(subscriberId, paths, callback) {
    return this._adapter.unsubscribe(subscriberId, paths, callback);
  },
  hasSubscriptions: function(subscriberId) {
    return this._adapter.hasSubscriptions(subscriberId);
  },
  subscribedToTxn: function(subscriberId, txn) {
    return this._adapter.subscribedToTxn(subscriberId, txn);
  }
};
PubSub._adapters = {};
PubSub._adapters.Redis = RedisAdapter = function(onMessage, options) {
  var db, event, host, makeCallback, namespace, port, redisOptions, subClient, subs, _fn, _i, _len, _onMessage, _ref, _ref2;
  redisOptions = (_ref = options.redis || {}, port = _ref.port, host = _ref.host, db = _ref.db, _ref);
  namespace = (db || 0) + '.';
  this._namespace = function(path) {
    return namespace + path;
  };
  if (!(this._publishClient = options.pubClient)) {
    this._publishClient = redis.createClient(port, host, redisOptions);
    if (db) {
      this._publishClient.select(db);
    }
  }
  if (!(this._subscribeClient = subClient = options.subClient)) {
    this._subscribeClient = redis.createClient(port, host, redisOptions);
  }
  this._subs = subs = {};
  this._subscriberSubs = {};
  if (options.debug) {
    _ref2 = ['subscribe', 'unsubscribe', 'psubscribe', 'punsubscribe'];
    _fn = function(event) {
      return subClient.on(event, function(path, count) {});
    };
    for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
      event = _ref2[_i];
      _fn(event);
    }
    subClient.on('message', function(channel, message) {
      return console.log("MESSAGE " + channel + " " + message);
    });
    subClient.on('pmessage', function(pattern, channel, message) {
      return console.log("PMESSAGE " + pattern + " " + channel + " " + message);
    });
    this.__publish = RedisAdapter.prototype.publish;
    this.publish = function(path, message) {
      console.log("PUBLISH " + (this._namespace(path)) + " " + (JSON.stringify(message)));
      return this.__publish(path, message);
    };
  }
  _onMessage = function(glob, path, message) {
    var key, re, subscriberId, subscribers, _ref3, _results;
    message = JSON.parse(message);
    if (subscribers = subs[glob]) {
      _results = [];
      for (key in subscribers) {
        _ref3 = subscribers[key], subscriberId = _ref3[0], re = _ref3[1];
        _results.push(re.test(path) ? onMessage(subscriberId, message) : void 0);
      }
      return _results;
    }
  };
  subClient.on('message', function(path, message) {
    return _onMessage(path, path, message);
  });
  subClient.on('pmessage', _onMessage);
  makeCallback = function(queue, events) {
    var event, fn, _j, _len2, _results;
    fn = function(path) {
      var callback, pending;
      if (pending = queue[path]) {
        if (callback = pending.shift()) {
          if (!--callback.__count) {
            return callback();
          }
        }
      }
    };
    _results = [];
    for (_j = 0, _len2 = events.length; _j < _len2; _j++) {
      event = events[_j];
      _results.push(subClient.on(event, fn));
    }
    return _results;
  };
  makeCallback(this._pendingSubscribe = {}, ['subscribe', 'psubscribe']);
  makeCallback(this._pendingUnsubscribe = {}, ['unsubscribe', 'punsubscribe']);
};
RedisAdapter.prototype = {
  subscribe: function(subscriberId, paths, callback) {
    var glob, globSubs, path, re, ss, subs, subscriberSubs, toAdd, _i, _len;
    if (subscriberId === void 0) {
      return;
    }
    subs = this._subs;
    subscriberSubs = this._subscriberSubs;
    toAdd = [];
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      path = paths[_i];
      path = this._namespace(path);
      glob = pathParser.glob(path);
      if (!(globSubs = subs[glob])) {
        subs[glob] = globSubs = {};
        toAdd.push(glob);
      }
      re = pathParser.regExp(path);
      globSubs["" + subscriberId + "$" + path] = [subscriberId, re];
      subscriberSubs[subscriberId] = ss = subscriberSubs[subscriberId] || {};
      ss[path] = re;
    }
    return handlePaths(toAdd, this._pendingSubscribe, this._subscribeClient, 'subscribe', 'psubscribe', callback);
  },
  publish: function(path, message) {
    path = this._namespace(path);
    return this._publishClient.publish(path, JSON.stringify(message));
  },
  unsubscribe: function(subscriberId, paths, callback) {
    var glob, globSubs, path, ss, subs, subscriberSubs, toRemove, _i, _len;
    if (subscriberId === void 0) {
      return;
    }
    if (typeof paths === 'function') {
      callback = paths;
      paths = null;
    }
    subscriberSubs = this._subscriberSubs;
    paths || (paths = subscriberSubs[subscriberId] || []);
    subs = this._subs;
    toRemove = [];
    for (_i = 0, _len = paths.length; _i < _len; _i++) {
      path = paths[_i];
      path = this._namespace(path);
      glob = pathParser.glob(path);
      if (globSubs = subs[glob]) {
        delete globSubs["" + subscriberId + "$" + path];
        if (!hasKeys(globSubs)) {
          toRemove.push(glob);
        }
      }
      ss = subscriberSubs[subscriberId];
      if (ss) {
        delete ss[path];
      }
    }
    return handlePaths(toRemove, this._pendingUnsubscribe, this._subscribeClient, 'unsubscribe', 'punsubscribe', callback);
  },
  hasSubscriptions: function(subscriberId) {
    return subscriberId in this._subscriberSubs;
  },
  subscribedToTxn: function(subscriberId, txn) {
    var p, path, re, _ref;
    path = this._namespace(transaction.path(txn));
    _ref = this._subscriberSubs[subscriberId];
    for (p in _ref) {
      re = _ref[p];
      if (p === path || re.test(path)) {
        return true;
      }
    }
    return false;
  }
};
handlePaths = function(paths, queue, client, pathFn, patternFn, callback) {
  var i, path, pending, _results;
  if (i = paths.length) {
    if (callback) {
      callback.__count = i;
    }
  } else {
    if (callback) {
      callback();
    }
  }
  _results = [];
  while (i--) {
    path = paths[i];
    if (pathParser.isGlob(path)) {
      client[patternFn](path);
    } else {
      client[pathFn](path);
    }
    _results.push(callback ? (queue[path] = pending = queue[path] || [], pending.push(callback)) : void 0);
  }
  return _results;
};
